FloatTable donnees;
float dmin, dmax;
int amin, amax;
int[] annees;
float traceX1, traceY1, traceX2, traceY2;
// La colonne de données actuellement utilisée.
int colonne = 0;
// Le nombre de colonnes.
int ncol;
// La police de caractères.
PFont police;

int intervalleAnnees = 10;
int intervalleVolume = 10;
int intervalleVolumeMineur = 5;

void setup() {
    size(720, 405);

    donnees = new FloatTable("lait-the-cafe.tsv");
    ncol = donnees.getColumnCount();        // Le nombre de colonnes.
    dmin = 0;
    dmax = ceil(donnees.getTableMax() / intervalleVolume) * intervalleVolume;
    annees = int(donnees.getRowNames());
    amin = annees[0];
    amax = annees[annees.length - 1];

    traceX1 = 50;
    traceY1 = 50;
    traceX2 = width - traceX1;
    traceY2 = height - traceY1;

    police = createFont("SansSerif", 20);
    textFont(police);

    smooth();
}

void draw() {
    background(224);

    fill(255);
    rectMode(CORNERS);
    noStroke();
    rect(traceX1, traceY1, traceX2, traceY2);
    
    dessineTitre(); // Dessine le titre.

    dessineAxeAnnees(); // Dessin de l'axe des années
    dessineAxeVolume(); // Dessin de l'axe des volumes

    strokeWeight(5);
    stroke(#5679C1);
    dessinePointsDonnees(colonne); // ! On utilise la colonne courante.
}

void dessinePointsDonnees(int col) {
    int lignes = donnees.getRowCount();
    for(int ligne = 0; ligne < lignes; ligne++) {
        if(donnees.isValid(ligne, col)) {
            float valeur = donnees.getFloat(ligne, col);
            float x = map(annees[ligne], amin, amax, traceX1, traceX2);
            float y = map(valeur, dmin, dmax, traceY2, traceY1);
            point(x, y);
        }
    }
}

void dessineAxeAnnees() {
    fill(0);
    textSize(10);
    textAlign(CENTER, TOP);

    // Des lignes fines en gris clair.
    stroke(224);
    strokeWeight(1);

    int lignes = donnees.getRowCount();
    for(int ligne = 0; ligne < lignes; ligne++) {
        if(annees[ligne] % intervalleAnnees == 0) {
            float x = map(annees[ligne], amin, amax, traceX1, traceX2);
            text(annees[ligne], x, traceY2 + 10);
            // Dessine les lignes.
            line(x, traceY1, x, traceY2);
        }
    }
    textAlign(0);
}

void dessineAxeVolume() {
    fill(0);
    textSize(10);
    stroke(128);
    strokeWeight(1);

    for(float v = dmin; v <= dmax; v+=intervalleVolumeMineur) {
        if(v % intervalleVolumeMineur == 0) {
            float y = map(v, dmin, dmax, traceY2, traceY1);
            if(v % intervalleVolume == 0) {
                if(v == dmin) {
                    textAlign(RIGHT, BOTTOM);
                } else if(v == dmax) {
                    textAlign(RIGHT, TOP);
                } else {
                    textAlign(RIGHT, CENTER);
                }
                text(floor(v), traceX1 - 10, y);
                line(traceX1 - 4, y, traceX1, y); // Tiret majeur.
            } else {
                line(traceX1 - 2, y, traceX1, y); // Tiret mineur.
            }
        }
    }

    textAlign(0);
}

void dessineTitre() {
    fill(0);
    textSize(20);
    textAlign(LEFT);
    text(donnees.getColumnName(colonne), traceX1, traceY1 - 10); 
}


void keyPressed() {
    if(keyCode == LEFT) {
        colonne = colonne == 0 ? ncol - 1 : (colonne - 1);
    }else if(keyCode == RIGHT) {
        colonne = (colonne + 1) % ncol;
    }
}